@@ -30,6 +30,7 @@ Generator for C/C++.
 from textwrap import dedent
 from pythran.tables import pythran_ward
 from pythran.spec import signatures_to_string
+from pythran.utils import quote_cxxstring
 
 __copyright__ = "Copyright (C) 2008 Andreas Kloeckner"
 
@@ -313,6 +314,15 @@ class Label(object):
     def generate(self):
         yield self.label + ':;'
 
+class LineInfo(object):
+    def __init__(self, filepath, lineno):
+        self.filepath = filepath
+        self.lineno = lineno
+
+    def generate(self):
+        if self.filename and self.lineno:
+            yield '#line {} {}'.format(self.lineno, self.filepath)
+
 
 class Statement(object):
     def __init__(self, text):
@@ -334,6 +344,16 @@ class AnnotatedStatement(object):
         for s in self.stmt.generate():
             yield s
 
+class StatementWithComments(object):
+    def __init__(self, stmt, comment):
+        self.stmt = stmt
+        self.comment = comment
+
+    def generate(self):
+        yield '// {}'.format(self.comment)
+        for s in self.stmt.generate():
+            yield s
+
 
 class ReturnStatement(Statement):
     def generate(self):
@@ -465,9 +485,7 @@ class PythonModule(object):
         self.metadata['moduledoc'] = moduledoc
 
     def docstring(self, doc):
-        return self.splitstring(dedent(doc).replace('"', '\\"')
-                                           .replace('\n', '\\n')
-                                           .replace('\r', '\\r'))
+        return self.splitstring(quote_cxxstring(dedent(doc)))
 
     def splitstring(self, doc):
         return '"{}"'.format('\\n""'.join(doc.split('\\n')))
