@@ -11,14 +11,17 @@ class ordered_set(object):
         self.values = list()
         self.unique_values = set()
         if elements is not None:
-            for elt in elements:
-                self.append(elt)
+            self.extend(elements)
 
     def append(self, value):
         if value not in self.unique_values:
             self.values.append(value)
             self.unique_values.add(value)
 
+    def extend(self, values):
+        for value in values:
+            self.append(value)
+
     def __iter__(self):
         return iter(self.values)
 
@@ -40,9 +43,7 @@ class TypeBuilder(object):
     >>> i_ty = builder.NamedType('int')
     >>> f_ty = builder.NamedType('float')
 
-    >>> l_ty + builder.NamedType('long')
-    long
-    >>> builder.NamedType('long') + builder.NamedType('char')
+    >>> builder.CombinedTypes(builder.NamedType('long'), builder.NamedType('char'))
     typename __combined<long,char>::type
 
     >>> builder.ArgumentType(4)
@@ -75,10 +76,10 @@ typename std::remove_reference<str>::type::iterator>::value_type>::type
     decltype(pythonic::builtins::getattr(\
 pythonic::types::attr::REAL{}, std::declval<complex>()))
 
-    >>> builder.ReturnType(builder.NamedType('math::cos'), [f_ty])
+    >>> builder.ReturnType(builder.NamedType('math::cos'), f_ty)
     decltype(std::declval<math::cos>()(std::declval<float>()))
 
-    >>> t = builder.TupleType([i_ty, builder.NamedType('str')])
+    >>> t = builder.TupleType(i_ty, builder.NamedType('str'))
     >>> builder.ElementType(1, t)
     typename std::tuple_element<1,typename std::remove_reference<\
 decltype(pythonic::types::make_tuple(std::declval<int>(), \
@@ -91,7 +92,7 @@ std::declval<str>()))>::type>::type
     >>> builder.SetType(builder.NamedType('int'))
     pythonic::types::set<int>
 
-    >>> builder.TupleType([i_ty, builder.NamedType('bool')])
+    >>> builder.TupleType(i_ty, builder.NamedType('bool'))
     decltype(pythonic::types::make_tuple(std::declval<int>(), \
 std::declval<bool>()))
 
@@ -105,7 +106,7 @@ std::declval<bool>()))
     indexable<int>
 
     >>> op = lambda x,y: x + '+' + y
-    >>> builder.ExpressionType(op, [l_ty, i_ty])
+    >>> builder.ExpressionType(op, l_ty, i_ty)
     decltype(std::declval<long>()+std::declval<int>())
     '''
 
@@ -145,13 +146,14 @@ std::declval<bool>()))
             def iscombined(self):
                 return False
 
-            def __add__(self, other):
-                if self is other:
-                    return self
-                return CombinedTypes(self, other)
+            def sgenerate(self):
+                return self.generate(lambda obj: obj.sgenerate())
 
             def __repr__(self):
-                return self.generate(str)
+                return self.sgenerate()  # for testing only
+
+            def __str__(self):
+                raise NotImplementedError("Call self.sgenerate() instead")
 
         class NamedType(Type):
             """
@@ -234,29 +236,12 @@ std::declval<bool>()))
             def iscombined(self):
                 return True
 
-            def __add__(self, other):
-                worklist = list(self.types)
-                visited = set()
-                while worklist:
-                    item = worklist.pop()
-                    if item is other:
-                        return self
-                    if item in visited:
-                        continue
-                    visited.add(item)
-                    if isinstance(item, CombinedTypes):
-                        worklist.extend(item.types)
-                return Type.__add__(self, other)
-
-            def __radd__(self, other):
-                return self.__add__(other)
-
             def generate(self, ctx):
                 # Degenerated trees may lead to very deep call stacks.
                 # In that case try hard to recover by cutting the tree
                 import sys
                 current_recursion_limit = sys.getrecursionlimit()
-                stypes = []
+                stypes = ordered_set()
                 for t in self.types:
                     try:
                         stypes.append(ctx(t))
@@ -399,7 +384,7 @@ std::declval<bool>()))
             '''
             Return type of a call with arguments
             '''
-            def __init__(self, ftype, args):
+            def __init__(self, ftype, *args):
                 super(ReturnType, self).__init__(ftype=ftype, args=args)
 
             def generate(self, ctx):
@@ -450,7 +435,7 @@ std::declval<bool>()))
             '''
             Type holding a tuple of stuffs of various types
             '''
-            def __init__(self, ofs):
+            def __init__(self, *ofs):
                 super(TupleType, self).__init__(ofs=ofs)
 
             def iscombined(self):
@@ -520,7 +505,7 @@ std::declval<bool>()))
             Result type of an operator call.
             """
 
-            def __init__(self, op, exprs):
+            def __init__(self, op, *exprs):
                 super(ExpressionType, self).__init__(op=op, exprs=exprs)
 
             def iscombined(self):
