@@ -272,6 +272,22 @@ namespace types
           "all slices are normalized");
     };
 
+    template <class... T1>
+    struct merge_gexpr<std::tuple<>, std::tuple<none_type, T1...>> {
+      template <size_t I, class S>
+      auto run(S const &s, std::tuple<> const &t0,
+               std::tuple<none_type, T1...> const &t1)
+          -> decltype(tuple_push_head(
+              std::get<0>(t1), merge_gexpr<std::tuple<>, std::tuple<T1...>>{}
+                                   .template run<I + 1>(s, t0, tuple_tail(t1))))
+      {
+        return tuple_push_head(
+            std::get<0>(t1),
+            merge_gexpr<std::tuple<>, std::tuple<T1...>>{}.template run<I + 1>(
+                s, t0, tuple_tail(t1)));
+      }
+    };
+
     template <class S0, class... T0, class... T1>
     struct merge_gexpr<std::tuple<S0, T0...>, std::tuple<none_type, T1...>> {
       template <size_t I, class S>
@@ -328,7 +344,7 @@ namespace types
       auto run(S const &s, std::tuple<S0, T0...> const &t0,
                std::tuple<long, T1...> const &t1)
           -> decltype(tuple_push_head(
-              std::get<0>(t1) * std::get<0>(t0).step + std::get<0>(t0).lower,
+              std::get<0>(t1),
               merge_gexpr<std::tuple<T0...>, std::tuple<T1...>>{}
                   .template run<I + 1>(s, tuple_tail(t0), tuple_tail(t1))))
       {
