@@ -90,7 +90,7 @@ def front_middle_end(module_name, code, optimizations=None, module_dir=None,
                      entry_points=None, report_times = False):
     """Front-end and middle-end compilation steps"""
 
-    pm = PassManager(module_name, module_dir)
+    pm = PassManager(module_name, module_dir, code)
 
     # front end
     ir, docstrings = frontend.parse(pm, code)
@@ -423,7 +423,7 @@ def compile_pythrancode(module_name, pythrancode, specs=None,
             logger.warning("Compilation error, "
                            "trying hard to find its origin...")
             error_checker()
-            logger.warning("Nop, I'm going to flood you with C++ errors!")
+            logger.warning("Nope, I'm going to flood you with C++ errors!")
             raise
 
     return output_file
@@ -451,7 +451,7 @@ def compile_pythranfile(file_path, output_file=None, module_name=None,
     Specify the output file:
 
     >>> import sysconfig
-    >>> ext = sysconfig.get_config_vars()["SO"]
+    >>> ext = sysconfig.get_config_vars()["EXT_SUFFIX"]
     >>> so_path = compile_pythranfile('pythran_test.py', output_file='foo'+ext)
     """
     if not output_file:
